local FONT = [=[Interface\AddOns\oUF_Senomar\calibri.ttf]=]
local FONTSIZE = 13
local TEXTURE = [=[Interface\AddOns\oUF_Senomar\minimalist]=]
local BACKDROP = {
	bgFile = TEXTURE, insets = {top = -1, bottom = -1, left = -1, right = -1}
}

oUF.TagEvents['seno:raidhealth'] = 'UNIT_HEALTH PLAYER_FLAGS_CHANGED'
oUF.Tags['seno:raidhealth'] = function(u)
	local min, max = UnitHealth(u), UnitHealthMax(u)
	return UnitIsDeadOrGhost(u) and format('|cffc8c8c8%s|r', oUF.Tags['dead'](u)) or not UnitIsConnected(u) and 'OFF' or UnitIsAFK(u) and 'AFK' or (min~=max) and format('|cffff8080%d|r', min-max) or max
end

oUF.TagEvents['seno:raidpower'] = 'UNIT_MANA UNIT_DISPLAYPOWER'
oUF.Tags['seno:raidpower'] = function(u)
	local min, max = UnitPower(u), UnitPowerMax(u)
	return (UnitPowerType(u) == 0 and min ~= max) and format('|cff%02x%02x%02x%.0f|r%%', 0, 144, 255, min/max*100)
end

oUF.Tags['assisticon'] = function(u) local i = GetRaidTargetIndex(u..'target') return i and ICON_LIST[i]..'22|t' end
oUF.Tags['assistname'] = function(u) return UnitName(u..'target') end
oUF.TagEvents['assisticon'] = 'UNIT_TARGET RAID_TARGET_UPDATE'
oUF.TagEvents['assistname'] = 'UNIT_TARGET UNIT_NAME_UPDATE'

-- ####################################

local function RaidMenu(self)
	if(self.unit:match('party')) then
		ToggleDropDownMenu(1, nil, _G['PartyMemberFrame'..self.id..'DropDown'], 'cursor')
	else
		FriendsDropDown.unit = self.unit
		FriendsDropDown.id = self.id
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
		ToggleDropDownMenu(1, nil, FriendsDropDown, 'cursor')
	end
end

local function Raid_Group(self, unit)
	self:SetBackdrop(BACKDROP)
	self:SetBackdropColor(0, 0, 0)

	self:RegisterForClicks('AnyDown')
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)

	self.menu = RaidMenu
	--self:SetAttribute('type2', 'menu')
	
	local health = CreateFrame('StatusBar', nil, self)
	health:SetAllPoints(self)
	health:SetStatusBarTexture(TEXTURE)
	health:SetStatusBarColor(0.2, 0.2, 0.2)
	--health.colorDisconnected = true
	self.Health = health
	
	local healthBG = health:CreateTexture(nil, 'BORDER')
	healthBG:SetAllPoints()
	healthBG:SetTexture(0.5, 0.5, 0.5)

	local LFDRole = self.Health:CreateTexture(nil, 'OVERLAY')
	LFDRole:SetPoint('LEFT', self, -18, 0)
	LFDRole:SetHeight(15)
	LFDRole:SetWidth(15)
	self.LFDRole = LFDRole

	local MasterLooter = self.Health:CreateTexture(nil, 'OVERLAY')
	MasterLooter:SetPoint('TOPLEFT', self, -5, 0)
	MasterLooter:SetHeight(12)
	MasterLooter:SetWidth(12)
	self.MasterLooter = MasterLooter

	local name = self.Health:CreateFontString(nil, 'OVERLAY')
	name:SetFont(FONT, FONTSIZE)
	name:SetShadowColor(0, 0, 0)
	name:SetShadowOffset(1, -1)
	name:SetPoint('LEFT', self, 'RIGHT', 5, 0)
	self:Tag(name, '[raidcolor][name< ][leader< ]|cff00ffff[- >assistname] [assisticon]')

	local health = self.Health:CreateFontString(nil, 'OVERLAY')
	health:SetFont(FONT, FONTSIZE)
	health:SetShadowColor(0, 0, 0)
	health:SetShadowOffset(1, -1)
	health:SetPoint('RIGHT', -3, 0)
	self:Tag(health, '[seno:raidhealth]')

	local power = self.Health:CreateFontString(nil, 'OVERLAY')
	power:SetFont(FONT, FONTSIZE)
	power:SetShadowColor(0, 0, 0)
	power:SetShadowOffset(1, -1)
	power:SetPoint('LEFT', 3, 0)
	self:Tag(power, '[seno:raidpower]')
	
	self.Range = {
		outsideAlpha = 0.4,
		insideAlpha = 1.0
	}
end

oUF:RegisterStyle('seno_raid', Raid_Group)

oUF:Factory(function(self)
	self:SetActiveStyle('seno_raid')
	local raid = {}
	local group = self:SpawnHeader(nil, nil, 'raid,party',
		'showParty', true,
		'showPlayer', true,
		'yOffset', -5,
		'showRaid', true,
		'groupFilter', '1',
		'oUF-initialConfigFunction', [[
			self:SetWidth(90)
			self:SetHeight(13)
		]]
	)
	table.insert(raid, group)
	group:SetPoint('TOPLEFT', UIParent, 20, -15)
	for i = 2, 8 do
		local group = self:SpawnHeader(nil, nil, 'raid,party',
			'yOffset', -5,
			'showRaid', true,
			'groupFilter', tostring(i),
			'oUF-initialConfigFunction', [[
				self:SetWidth(90)
				self:SetHeight(13)
			]]
		)
		group:SetPoint('TOP', raid[i-1], 'BOTTOM', 0, -20)
		table.insert(raid, group)
	end
end)
